/*
 * E-Zdrojov kd.c
 *
 * Created: 16.04.2019 13:40:18
 * Author : Milo Mldek
 */ 

/*
m0-zablesky LED
m1-po stisku se motor rozebehne, bezi dokud neni stisknuto tlacitko znovu
m2-po stisku tlacitka se motor rozebehne, bezi dokud neni tlacitko pusteno
m3-po stisku tlacitka se motor rozebehne, bezi dokud neni tlacitko pusteno,
	zvysena povolena hodnota proudu a strida ovladaciho napeti



*/

#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>
#include <avr/interrupt.h>


#define SPEED  240	//hodnota PWM ucujici rychlost otaceni motoru
#define SPEED_S 255	//hodnota PWM ucujici rychlost otaceni motoru pro mode == 3
#define MAX_VOLTAGE 35	//hodnota ADC nad kterou motor stoji
#define MAX_VOLTAGE_S 35	//hodnota ADC nad kterou motor stoji pro mode == 3
#define T_START 150 //prodleva mezi spustenim motoru a zacatkem hlidani proudu
#define TW_END 100	//cas k ustaleni po zastaveni v koncove poloze
#define T_MOVE_BACK 400 //doba vraceni se v koncove poloze
#define TW_RC 50	//doba k ustalen v prubehu zpracovani rc
#define TW_RC_DONE 100	//doba k ustaleno po zpracovani signalu rc
#define TW_VREF 50	//cas ustaleni vnitrni napetove reference
#define T_FLICK_ON 100 //doba rozsviceni LED pri blikani
#define T_FLICK_OFF_M0 900 //doba zhasnuti LED mode0
#define T_FLICK_OFF_MT 100 //doba zhasnuti LED mode_test
#define TW_ANTI_BOUNCE 200 //doba pro ustaleni zakmitu
#define TW_STOP 150 //ustaleni po zastaven motoru

void setup(void);
void get_mode(void);
void mode0(void);
void mode_problem(void);
void scan_tl(void);
void scan_tl1_m1(void);
void scan_tl2_m1(void);
void scan_tl1_m2(void);
void scan_tl2_m2(void);
void scan_rc(void);
void scan_rc1_m1(void);
void scan_rc2_m1(void);
void scan_rc1_m2(void);
void scan_rc2_m2(void);
void runA(void);
void runB(void);
void stop(void);


volatile unsigned char runOKdA = 1;
volatile unsigned char runOKdB = 1;
volatile unsigned int voltage = 0;
volatile unsigned int max_voltage = 0;

unsigned int mode;

unsigned char tl1_state;
unsigned char prev_tl1_state;
unsigned char tl2_state;
unsigned char prev_tl2_state;
unsigned char rc1_state;
unsigned char rc2_state;



//koncovy spinac
ISR(INT0_vect)
{
	PORTB |= (1<<PB3);
	cli();
	stop();
	_delay_ms(TW_END);
	runA();
	_delay_ms(T_MOVE_BACK);	//prodleva
	stop();
	runOKdB = 1;
	PORTB &= ~(1<<PB3);
}
//koncovy spinac
ISR(INT1_vect)
{
	PORTB |= (1<<PB3);
	cli();
	stop();
	_delay_ms(TW_END);	//prodleva
	runB();
	_delay_ms(T_MOVE_BACK);	//prodleva
	stop();
	runOKdA = 1;
	PORTB &= ~(1<<PB3);
}
//dalkove ovladani
ISR(INT2_vect){
	PORTB |= (1<<PB3);
	scan_rc();
	_delay_ms(TW_RC_DONE);
	PORTB &= ~(1<<PB3);
}
//hlidani proudu
ISR(ADC_vect){
	voltage = ADC;
	if (voltage > max_voltage)
	{
		PORTB |= (1<<PB3);
		stop();
		runOKdA = 1;
		runOKdB = 1;
	}
	else
	{
		PORTB &= ~(1<<PB3);
	}
}


int main(void)
{
	setup();
	
	while (1)
	{
		if (runOKdA & runOKdB)
		{
			get_mode();
			switch (mode)
			{
			case 0:
				mode0();
				break;
			case 1 ... 2:
				
				OCR1B = SPEED;
				max_voltage = MAX_VOLTAGE;
				
				GICR |= (1<<INT0)|(1<<INT1)|(1<<INT2); //povoleni externich preruseni
				break;
			case 3:
				OCR1B = SPEED_S;		
				max_voltage = MAX_VOLTAGE_S;
				
				GICR |= (1<<INT0)|(1<<INT1)|(1<<INT2); //povoleni externich preruseni
				break;
			default:
				GICR |= ~(1<<INT2); //zakazani INT2
				mode_problem();
				
				
			
			}
		}
		scan_tl();

		sei();
	}
	return 0;
}

void setup(void){
	//nastaveni vstupu
	
	DDRC &= ~((1 << PC6)|(1 << PC7)); //nastaveni vstupu dip2 a dip1
	PORTC &= ~((1<<PC6)|(PC7));	//neneastaveni pull-up pro dip2 a dip1
	DDRB &= ~((1<<PB1)|(1<<PB0));	//nastaveni vstupu tl2 a tl1
	PORTB |= (1<<PB1)|(1<<PB0);  //nastaveni pull-up rezistoru tl2 a tl1
	DDRD &= ~((1 << PD3)|(1 << PD2)); //nastaveni vstupu ks1 a ks2
	DDRB &= ~(1<<PB3);	//nastaveni vstupu vt
	PORTB &= ~(1<<PB3);	//neneastaveni pull-up pro vt
	DDRC &= ~((1<<PC5)|(1<<PC4));	//nastaveni vystupu rc1 a rc2
	PORTC &= ~((1<<PC5)|(PC4));	//neneastaveni pull-up pro rc1 a rc2
	
	//nastaveni preruseni
	MCUCR &= ~((1<<ISC11)|(1<<ISC10)|(1<<ISC01)|(1<<ISC00));	//zadost preruseni pri urovni 0
	MCUCSR |= (1<<ISC2); //pozadavek na preruseni INT2 s nabeznou hranou
	GICR |= (1<<INT0)|(1<<INT1)|(1<<INT2); //povoleni externich preruseni
	
	//nastaveni 8-bit Phase Correct PWM, non-inverted
	TCCR1A|=(1<<COM1B1)|(1<<WGM10);
	TCCR1B|=(1<<CS10);

	//nastaveni vystupu ovladajicich motor
	DDRD |=(1<<PD4);	//Enable
	DDRD |=(1<<PD5);	//In1
	DDRD |=(1<<PD6);	//In2

	//nastaveni vystupu LED
	DDRB |=(1<<PB3);
	
	//nastaveni ADC
	ADMUX |= (1<<REFS1)|(1<<REFS0); //nastaveni interni reference 2,56 V
	_delay_ms(TW_VREF); //ustaleni interni reference
	ADMUX &= ~((1<<MUX4)|(1<<MUX3)|(1<<MUX2)|(1<<MUX1)); //nastaveni kanalu ADC0
	ADMUX &= ~(1<<ADLAR);	//zarovnni vsledk vpravo
	ADCSRA |= (1<<ADEN)|(1<<ADIE);	//povolen pevodnku, povolen peruen po dokonen pevodu
	ADCSRA |= (1<<ADATE);	//povoleni Auto Triggering
	ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);	//nastaveni delicky kmitocu prevodniku na 128
	SFIOR &= 0x1f;	//nastaveni Free-Running mode
	
	tl1_state = 1;	// tlacitko neni stisknute
	tl2_state = 2;	// tlacitko neni stisknute
	

}

void get_mode(void){
	mode = PINC;
	mode &= 192;
	mode = (mode >> 6);
}

void mode0(void){
	cli();
	ADCSRA &= ~((1<<ADSC)|(1<<ADIE)|(1<<ADEN)); //zastaven konverze
	PORTB |= (1<<PB3);
	_delay_ms(T_FLICK_ON);
	PORTB &= ~(1<<PB3);
	_delay_ms(T_FLICK_OFF_M0);
	
	main();
}

void mode_problem(void){
	cli();
	ADCSRA &= ~((1<<ADSC)|(1<<ADIE)|(1<<ADEN)); //zastaven konverze
	PORTB |= (1<<PB3);
	_delay_ms(T_FLICK_ON);
	PORTB &= ~(1<<PB3);
	_delay_ms(T_FLICK_OFF_MT);
	main();
}


void scan_tl(void){
	prev_tl1_state = tl1_state;	// ulozeni predchoziho stavu
	prev_tl2_state = tl2_state;	// ulozeni predchoziho stavu
	tl1_state = PINB; //ulozeni stavu tl1
	tl2_state = tl1_state;
	tl1_state &= 1;
	tl2_state &= 2;
	switch(mode){
		case 1:		scan_tl1_m1();
		scan_tl2_m1();
		break;
		case 2:		scan_tl1_m2();
		scan_tl2_m2();
		break;
		case 3:		scan_tl1_m2();
		scan_tl2_m2();
		break;
		default:	mode_problem();
	}

}

void scan_tl1_m1(void){
	if (tl1_state == 0)	//soucasny stav log 0
	{if (prev_tl1_state == 0) //stisknuti tlacitka, prechod z log 1 na 0
		{
			
			if (runOKdA)
			{
				runA();
				runOKdB = 0;
			}
			else
			{
				stop();
				runOKdB = 1;
			}
			_delay_ms(TW_ANTI_BOUNCE);
			
		}
	}
	else			//soucasny stav log 1
	{if (prev_tl1_state == 0)	//uvolneni tlacitka prechod log hodnot z 0 na 1
		{
			runOKdA = ~runOKdA;
			runOKdA = (runOKdA&1);
			_delay_ms(TW_ANTI_BOUNCE);
		}
	}
}

void scan_tl2_m1(void){
	if (tl2_state == 0)	//soucasny stav log 0
	{if (prev_tl2_state == 0) //stisknuti tlacitka, prechod z log 1 na 0
		{
			if (runOKdB)
			{
				runB();
				runOKdA = 0;
			}
			else
			{
				stop();
				runOKdA = 1;
			}
			_delay_ms(TW_ANTI_BOUNCE);
		}
	}
	else			//soucasny stav log 1
	{if (prev_tl2_state == 0)	//uvolneni tlacitka prechod log hodnot z 0 na 1
		{
			runOKdB = ~runOKdB;
			runOKdB = (runOKdB&1);
			_delay_ms(TW_ANTI_BOUNCE);
		}
	}
}

void scan_tl1_m2(void){
	if (tl1_state == 0)	//soucasny stav log 0
	{if (prev_tl1_state == 0) //stisknuti tlacitka, prechod z log 1 na 0
		{
			runA();
			_delay_ms(TW_ANTI_BOUNCE);
		}
	}
	else			//soucasny stav log 1
	{if (prev_tl1_state == 0)	//uvolneni tlacitka prechod log hodnot z 0 na 1
		{
			stop();
			_delay_ms(TW_ANTI_BOUNCE);
		}
	}
}

void scan_tl2_m2(void){
	if (tl2_state == 0)	//soucasny stav log 0
	{if (prev_tl2_state == 0) //stisknuti tlacitka, prechod z log 1 na 0
		{
			runB();
			_delay_ms(TW_ANTI_BOUNCE);
		}
	}
	else			//soucasny stav log 1
	{if (prev_tl2_state == 0)	//uvolneni tlacitka prechod log hodnot z 0 na 1
		{
			stop();
			_delay_ms(TW_ANTI_BOUNCE);
		}
	}
}

void scan_rc(void){
	sei();
	rc1_state = PINC; //ulozeni stavu rc1 a rc2
	rc2_state = rc1_state;
	rc1_state &= (1<<PC5);	//rc1 pripojeno k PC5
	rc2_state &= (1<<PC4);	//rc2 pripojeno k PC4
	switch(mode){
		case 1:
			scan_rc1_m1();
			scan_rc2_m1();
		break;
		case 2:
			scan_rc1_m2();
			scan_rc2_m2();
		break;
		default:	mode_problem();
	
	}
}

void scan_rc1_m1(void){
	if (rc1_state != 0)
	{
		if (runOKdA)
		{
			runA();
			runOKdB = 0;
			runOKdA = 0;
		}
		else
		{
			stop();
			runOKdA = 1;
			runOKdB = 1;
		}
		_delay_ms(TW_RC);
		
		
	}
	
}

void scan_rc2_m1(void){
	if (rc2_state != 0)	//soucasny stav log 0
	{if (runOKdB)
		{
			runB();
			runOKdA = 0;
			runOKdB = 0;
		}
		else
		{
			stop();
			runOKdA = 1;
			runOKdB = 1;
		}
		_delay_ms(TW_RC);
	}
}

void scan_rc1_m2(void){
	if (rc1_state != 0)
	{
		runA();
		runOKdB = 0;
		runOKdA = 0;
		while (PINC & (1<<PC5))
		{
		}
		stop();
		runOKdA = 1;
		runOKdB = 1;
		
		_delay_ms(TW_RC);
		
		
	}
	
}

void scan_rc2_m2(void){
	if (rc2_state != 0)	//soucasny stav log 0
	runB();
	runOKdB = 0;
	runOKdA = 0;
	while (PINC & (1<<PC4))
	{
	}
	stop();
	runOKdA = 1;
	runOKdB = 1;
	_delay_ms(TW_RC);
}

void runA(void){
	PORTD |= (1<<PD5);
	PORTD &= ~(1<<PD6);
	_delay_ms(T_START);	//prodleva
	ADCSRA |= (1<<ADSC)|(1<<ADIE)|(1<<ADEN);	//spusteni konverze ADC
}

void runB(void){
	PORTD &= ~(1<<PD5);
	PORTD |= (1<<PD6);
	
	_delay_ms(T_START);	//prodleva
	ADCSRA |= (1<<ADSC)|(1<<ADIE)|(1<<ADEN);	//spusteni konverze ADC
}

void stop(void){
	PORTD &= ~(1<<PD5);
	PORTD &= ~(1<<PD6);
	ADCSRA &= ~((1<<ADSC)|(1<<ADIE)|(1<<ADEN)); //zastaven konverze ADC
	_delay_ms(TW_STOP);	//prodleva
}
